@@ -19,10 +19,17 @@ Radars
 .. autosummary::
     kkpy.util.dbzmean
 
+Microphysics
+---------------
+.. autosummary::
+    kkpy.util.vel_atlas
+    kkpy.util.calc_dsdmoments
+
 Maps
 -------
 .. autosummary::
     kkpy.util.proj_dfs
+    kkpy.util.proj_icepop
     kkpy.util.dist_bearing
 
 Spatial calculations
@@ -32,11 +39,11 @@ Spatial calculations
     kkpy.util.cross_section_2d
     
 ICE-POP 2018
--------------
+--------------
 .. autosummary::
     kkpy.util.icepop_events
     kkpy.util.icepop_sites
-    kkpy.util.icepop_
+    kkpy.util.icepop_extent
 
 Miscellaneous
 ---------------
@@ -44,7 +51,10 @@ Miscellaneous
     kkpy.util.std2d
     kkpy.util.nanstd2d
     kkpy.util.nanconvolve2d
-"""                                                   
+    kkpy.util.derivative
+    kkpy.util.stats
+    kkpy.util.summary
+"""
 import numpy as np
 
 def wind2uv(wd=None, ws=None, radians=False, knots=False):
@@ -249,7 +259,7 @@ def cross_section_2d(xy0, xy1, x2d, y2d, value2d, linewidth=1, along='x', reduce
     
     # get xaxis
     ixlen, iylen = iend - istart
-    ixsize = np.int(np.ceil(np.hypot(ixlen, iylen) + 1))
+    ixsize = int(np.ceil(np.hypot(ixlen, iylen) + 1))
     if along in 'x':
         xaxis = np.linspace(xy0[0], xy1[0], ixsize)
     else:
@@ -275,6 +285,7 @@ def proj_dfs():
     
     Examples
     ---------
+    >>> import cartopy.crs as ccrs
     >>> ax = plt.subplot(proj=kkpy.util.proj_dfs())
     >>> ax.scatter([126], [38], transform=ccrs.PlateCarree())
     >>> plt.show()
@@ -298,6 +309,37 @@ def proj_dfs():
                                  globe=globe)
     return proj
 
+def proj_icepop():
+    """
+    Return a lambert conformal conic projection for ICE-POP 2018 domain.
+    
+    Examples
+    ---------
+    >>> import cartopy.crs as ccrs
+    >>> ax = plt.subplot(proj=kkpy.util.proj_icepop())
+    >>> ax.scatter([129], [37.7], transform=ccrs.PlateCarree())
+    >>> ax.set_extent(kkpy.util.icepop_extent(), crs=ccrs.PlateCarree())
+    >>> plt.show()
+    
+    Returns
+    ---------
+    proj : cartopy projection
+        Return a map projection of DFS.
+    """
+    import cartopy.crs as ccrs
+    
+    globe = ccrs.Globe(ellipse=None,
+                       semimajor_axis=6371008.77,
+                       semiminor_axis=6371008.77)
+    
+    proj = ccrs.LambertConformal(central_longitude=128.75,
+                                 central_latitude=37.65,
+                                 standard_parallels=(30,60),
+                                 false_easting=75000,
+                                 false_northing=75000,
+                                 globe=globe)
+    return proj
+
 def dist_bearing(lonlat0, lonlat1, radians=False):
     """
     Get distance [km] and bearing [deg] between two lon/lat points.
@@ -595,18 +637,32 @@ def icepop_events(KST=False):
         Return a dictionary of array containing start and finish datetime for each event.
     
     """
+    import datetime
     
-    events = {
-        1: [datetime.datetime(2017,11,24,20), datetime.datetime(2017,11,25,19)],
-        2: [datetime.datetime(2017,12,23,20), datetime.datetime(2017,12,25, 3)],
-        3: [datetime.datetime(2018, 1,22, 3), datetime.datetime(2018, 1,23, 0)],
-        4: [datetime.datetime(2018, 2,27,23), datetime.datetime(2018, 3, 1, 3)],
-        5: [datetime.datetime(2018, 3, 4, 8), datetime.datetime(2018, 3, 5,10)],
-        6: [datetime.datetime(2018, 3, 7, 5), datetime.datetime(2018, 3, 9, 3)],
-        7: [datetime.datetime(2018, 3,14,19), datetime.datetime(2018, 3,16,12)],
-        8: [datetime.datetime(2018, 3,18,10), datetime.datetime(2018, 3,19, 9)],
-        9: [datetime.datetime(2018, 3,19,21), datetime.datetime(2018, 3,21,14)],
-    }
+    if not KST:
+        events = {
+            1: [datetime.datetime(2017,11,24,20), datetime.datetime(2017,11,25,19)],
+            2: [datetime.datetime(2017,12,23,20), datetime.datetime(2017,12,25, 3)],
+            3: [datetime.datetime(2018, 1,22, 3), datetime.datetime(2018, 1,23, 0)],
+            4: [datetime.datetime(2018, 2,27,23), datetime.datetime(2018, 3, 1, 3)],
+            5: [datetime.datetime(2018, 3, 4, 8), datetime.datetime(2018, 3, 5,10)],
+            6: [datetime.datetime(2018, 3, 7, 5), datetime.datetime(2018, 3, 9, 3)],
+            7: [datetime.datetime(2018, 3,14,19), datetime.datetime(2018, 3,16,12)],
+            8: [datetime.datetime(2018, 3,18,10), datetime.datetime(2018, 3,19, 9)],
+            9: [datetime.datetime(2018, 3,19,21), datetime.datetime(2018, 3,21,14)],
+        }
+    else:
+        events = {
+            1: [datetime.datetime(2017,11,25, 5), datetime.datetime(2017,11,26, 4)],
+            2: [datetime.datetime(2017,12,24, 5), datetime.datetime(2017,12,25,12)],
+            3: [datetime.datetime(2018, 1,22,12), datetime.datetime(2018, 1,23, 9)],
+            4: [datetime.datetime(2018, 2,28, 8), datetime.datetime(2018, 3, 1,12)],
+            5: [datetime.datetime(2018, 3, 4,17), datetime.datetime(2018, 3, 5,19)],
+            6: [datetime.datetime(2018, 3, 7,14), datetime.datetime(2018, 3, 9,12)],
+            7: [datetime.datetime(2018, 3,15, 4), datetime.datetime(2018, 3,16,21)],
+            8: [datetime.datetime(2018, 3,18,19), datetime.datetime(2018, 3,19,18)],
+            9: [datetime.datetime(2018, 3,20, 6), datetime.datetime(2018, 3,21,23)],
+        }
     
     return events
 
@@ -616,33 +672,284 @@ def icepop_sites():
     
     Examples
     ---------
-    >>> dts = kkpy.util.icepop_events()
-    >>> for eventno in np.arange(9)+1:
-    >>>     dt = dts[eventno]
-    >>>     print(f'eventno = {eventno}, start = {dt[0]}, end = {dt[1]}')
+    >>> # Plotting in Lon/Lat coordinates
+    >>> for lon, lat, hgt, site in kkpy.util.icepop_sites():
+    >>>     plt.plot(lon, lat, marker='o', color='red', markersize=3, alpha=0.5, transform=ccrs.PlateCarree())
+    >>>     plt.text(lon+0.01, lat, site, verticalalignment='center', fontsize=10, transform=ccrs.PlateCarree())
+    
+    >>> # Plotting in Lon/Hgt coordinates
+    >>> for lon, lat, hgt, site in kkpy.util.icepop_sites():
+    >>>     plt.plot(lon, hgt/1e3, marker='o', color='red', markersize=3, alpha=0.5)
+    >>>     plt.text(lon+0.01, hgt/1e3, site, verticalalignment='center', fontsize=10)
     
-    Parameters
-    ----------
-    KST : boolean, optional
-        True if return the datetime in KST, rather than UTC.
-        
     Returns
     ---------
-    dict_datetimes : dictionary
-        Return a dictionary of array containing start and finish datetime for each event.
+    tuple_sites : tuple
+        Return a tuple containing longitude, latitude, height, and sitename for each ICE-POP 2018 supersite.
     
     """
     
-    events = {
-        1: [datetime.datetime(2017,11,24,20), datetime.datetime(2017,11,25,19)],
-        2: [datetime.datetime(2017,12,23,20), datetime.datetime(2017,12,25, 3)],
-        3: [datetime.datetime(2018, 1,22, 3), datetime.datetime(2018, 1,23, 0)],
-        4: [datetime.datetime(2018, 2,27,23), datetime.datetime(2018, 3, 1, 3)],
-        5: [datetime.datetime(2018, 3, 4, 8), datetime.datetime(2018, 3, 5,10)],
-        6: [datetime.datetime(2018, 3, 7, 5), datetime.datetime(2018, 3, 9, 3)],
-        7: [datetime.datetime(2018, 3,14,19), datetime.datetime(2018, 3,16,12)],
-        8: [datetime.datetime(2018, 3,18,10), datetime.datetime(2018, 3,19, 9)],
-        9: [datetime.datetime(2018, 3,19,21), datetime.datetime(2018, 3,21,14)],
-    }
+    sites = (
+        (128.866858, 37.770897,  36, 'GWU'),
+        (128.805847, 37.738157, 175, 'BKC'),
+        (128.758636, 37.686953, 855, 'CPO'),
+        (128.718825, 37.677331, 773, 'DGW'),
+        (128.699611, 37.665208, 789, 'MHS'),
+        (128.670494, 37.643342, 772, 'YPO'),
+        (128.564700, 38.250900,  18, 'SCW'),
+        (128.629700, 38.087200,   4, 'YYO'),
+        (128.540700, 38.007400, 146, 'YDO'),
+        (128.821100, 37.898300,  10, 'JMO'),
+        (129.028900, 37.613500,  58, 'OGO'),
+        (129.124300, 37.507100,  40, 'DHW'),
+        (128.377600, 37.562000, 532, 'MOO'),
+        (128.394600, 37.377900, 303, 'PCO')
+    )
     
-    return events
+    return sites
+
+def icepop_extent():
+    """
+    Get extent of ICE-POP 2018 domain.
+    
+    Examples
+    ---------
+    >>> ax.set_extent(kkpy.util.icepop_extent(), crs=ccrs.PlateCarree())
+    
+    Returns
+    ---------
+    list_extent : list
+        Return a list containing lon0, lon1, lat0, and lat1 of default ICE-POP 2018 domain.
+    
+    """
+    return [128, 129.5, 37, 38.3]
+
+def vel_atlas(D):
+    """
+    Get Atlas et al. (1973) fall velocity.
+    
+    Examples
+    ---------
+    >>> V_arr = kkpy.util.vel_atlas(D_arr)
+    
+    Parameters
+    ----------
+    D : array_like
+        Array containing the diameter in mm.
+        
+    Returns
+    ---------
+    V : array_like
+        Return a fall velocity of raindrop for a given diameter.
+    """
+    return 9.65 - 10.3 * np.exp(-0.6 * D)
+
+def calc_dsdmoments(ND, D, dD, VD=None, Vatlas=False):
+    """
+    Get DSD moments from drop size distribution.
+    
+    Examples
+    ---------
+    >>> tdvd = kkpy.util.calc_dsdmoments(ND_arr, D_arr, dD_arr, VD=VD_arr)
+    
+    >>> poss = kkpy.util.calc_dsdmoments(ND_arr, D_arr, dD_arr, Vatlas=True)
+    
+    Parameters
+    ----------
+    ND : array_like
+        Array containing the number concentration in mm-1 m-3. If 2D array, the diameter should be at the first axis (i.e. axis = 0).
+    D : array_like
+        Array containing the median of each diameter channel in mm.
+    dD : array_like
+        Array containing the spread of each diameter channel in mm.
+    VD : array_like, optional
+        Array containing the mean velocity of each diameter channel in m s-1. Vatlas should be False if VD contains data.
+    Vatlas : boolean, optional
+        True if VD is replaced by the velocity-diameter relationship of Atlas et al. (1973).
+        
+    Returns
+    ---------
+    dict_moments : dictionary
+        Return a dictionary containing the DSD moments: z[mm6 m-3], Z[dBZ], and R[mm hr-1].
+    """
+    
+    if Vatlas:
+        VD = vel_atlas(D)
+    
+    dict_ = {}
+    if ND.ndim == 2:
+        dict_['z'] = np.sum(ND*(D**6)*dD, axis=1) # mm6 m-3
+        dict_['R'] = np.pi*6*1e-4*np.sum(ND*VD*(D**3)*dD, axis=1) # mm hr-1
+    else:
+        dict_['z'] = np.sum(ND*(D**6)*dD) # mm6 m-3
+        dict_['R'] = np.pi*6*1e-4*np.sum(ND*VD*(D**3)*dD) # mm hr-1
+    
+    dict_['Z'] = 10*np.log10(dict_['z']) # dBZ
+    
+    return dict_
+
+def stats(x, y,
+          fmtbias='.3f', fmtrmse='.3f',
+          fmtstd='.3f', fmtcorr='.3f'):
+    """
+    Get performance matrix BIAS, RMSE, STD, and CORR values and its f-string.
+    
+    Examples
+    ---------
+    >>> stats, str_stat = kkpy.util.stats(xarr, yarr)
+    >>> print(stats)
+    >>> plt.text(3, 5, str_stat)
+    
+    >>> stats, str_stat = kkpy.util.stats(xarr, yarr, fmtbias='.1f', fmtcorr='.2f')
+    
+    Parameters
+    ----------
+    x : array_like
+        Array containing multiple variables and observations.
+    y : array_like
+        Array containing multiple variables and observations. The shape should be same as **x**.
+    fmtbias : str, optional
+        String format for BIAS. Default is '.3f'.
+    fmtrmse : str, optional
+        String format for RMSE. Default is '.3f'.        
+    fmtstd : str, optional
+        String format for STD. Default is '.3f'.        
+    fmtcorr : str, optional
+        String format for CORR. Default is '.3f'.        
+
+    Returns
+    ---------
+    dict_moments : dictionary
+        Return a dictionary containing the DSD moments: z[mm6 m-3], Z[dBZ], and R[mm hr-1].
+    """
+    import pandas as pd
+    
+    bias = np.nanmean(y - x)
+    rmse = np.nanmean((y - x) ** 2)**0.5
+    std = np.nanstd(y - x)
+    corr = pd.Series(x).corr(pd.Series(y))
+    
+    stats = [bias, rmse, std, corr]
+    str_stat = f'BIAS={bias:.3f}\nRMSE={rmse:.3f}\nSTD={std:.3f}\nCORR={corr:.3f}'
+    
+    return stats, str_stat
+
+def summary(arr, cnt_print=10):
+    """
+    Get summary of the array.
+    
+    Parameters
+    ----------
+    arr : array_like
+        Array containing multiple variables and observations.
+    cnt_print : int, optional
+        The number of elements to print.
+    """
+    import pandas as pd
+    import matplotlib.pyplot as plt
+    
+    print(pd.Series(arr).describe())
+    cnt = arr.size
+    count_nan = np.count_nonzero(np.isnan(arr))
+    print(f'\nNans   {count_nan} ({count_nan/arr.size:.1f} %)\n')
+    
+    if cnt > cnt_print:
+        print(f'First {cnt_print} values:\n {arr[:cnt_print]}\n')
+        print(f'Last {cnt_print} values:\n {arr[-cnt_print:]}\n')
+        
+        arr = arr[~np.isnan(arr)]
+        arr_sorted = np.sort(arr)
+        print(f'Lowest {cnt_print} values:\n {arr_sorted[:cnt_print]}\n')
+        print(f'Highest {cnt_print} values:\n {arr_sorted[-cnt_print:]}\n')
+        arr_unique = np.unique(arr)
+        print(f'Lowest {cnt_print} unique values:\n {arr_unique[:cnt_print]}\n')
+        print(f'Highest {cnt_print} unique values:\n {arr_unique[-cnt_print:]}\n')
+        
+        hist = plt.hist(arr)
+        print('Histograms')
+        for y, x in zip(hist[0], hist[1]):
+            print(f'{x:.3f} -- {y:.0f}')
+        
+    return
+
+def derivative(var, cnt_x, pixelsize=1, axis=0):
+    """
+    Get derivative of 1D or 2D array.
+    
+    Examples
+    ---------
+    >>> # Derivative of 1D array with a window size of 5.
+    >>> # Note that the horizontal resolution of xarr is 0.1.
+    >>> xarr = np.arange(100)*0.1
+    >>> yarr = np.exp(xarr)
+    >>> dydx = kkpy.util.derivative(yarr, 5, pixelsize=0.1)
+    
+    >>> # This example gives the derivative of arr2d along x axis with a window size of 20.
+    >>> # Assume that you have an arr2d[iy,ix] with horizontal resolution of 3 km.
+    >>> dzdx = kkpy.util.derivative(arr2d, 20, pixelsize=3000, axis=1)
+    
+    Parameters
+    ----------
+    var : array_like
+        Array containing multiple variables and observations.
+    cnt_x : int
+        The number of the pixel for derivative. This should be odd number. The derivative will be (Y[i+N/2]-Y[i-N/2]) / (N-1).
+    pixelsize : float, optional
+        The size of one pixel of x axis. If default (1), the derivative means the rate of change on an interval of **one pixel** with resolution of 1.
+    axis : int, optional
+        Axis along which the derivatives are computed. Default is 0.
+        
+    Returns
+    ---------
+    derivative : array_like
+        Return a derivative for a given array.
+    """
+    import astropy.convolution
+
+    if cnt_x % 2 != 1:
+        raise ValueError('cnt_x must be odd.')
+    
+    if var.ndim > 2:
+        raise ValueError('dimension size of var should be <= 2')
+
+    kernel = np.zeros(cnt_x)
+    kernel[0] = 1
+    kernel[-1] = -1
+    
+    if var.ndim == 2:
+        if axis == 0:
+            kernel = kernel[:,np.newaxis]
+        if axis == 1:
+            kernel = kernel[np.newaxis,:]
+    
+    result = astropy.convolution.convolve(
+            var,
+            kernel,
+            boundary='extend',
+            normalize_kernel=False,
+            nan_treatment='fill',
+            fill_value=np.nan,
+            preserve_nan=True
+        ) / (cnt_x-1)
+    
+    if var.ndim == 1:
+        for j in np.arange(int(cnt_x/2)+1):
+            result[j] = (var[j+int(cnt_x/2)]-var[0]) / (j+int(cnt_x/2))
+        for j in np.arange(var.size-1-int(cnt_x/2), var.size):
+            result[j] = (var[-1]-var[j-int(cnt_x/2)]) / (var.size-j+int(cnt_x/2)-1)
+    
+    if var.ndim == 2:
+        if axis == 0:
+            for j in np.arange(int(cnt_x/2)+1):
+                result[j,:] = (var[j+int(cnt_x/2),:]-var[0,:]) / (j+int(cnt_x/2))
+            for j in np.arange(var.shape[0]-1-int(cnt_x/2), var.shape[0]):
+                result[j,:] = (var[-1,:]-var[j-int(cnt_x/2),:]) / (var.shape[0]-j+int(cnt_x/2)-1)
+        if axis == 1:
+            for j in np.arange(int(cnt_x/2)+1):
+                result[:,j] = (var[:,j+int(cnt_x/2)]-var[:,0]) / (j+int(cnt_x/2))
+            for j in np.arange(var.shape[1]-1-int(cnt_x/2), var.shape[1]):
+                result[:,j] = (var[:,-1]-var[:,j-int(cnt_x/2)]) / (var.shape[1]-j+int(cnt_x/2)-1)
+    
+    derivative = result / pixelsize
+    
+    return derivative
